function love.load()	

	class = require '30log'

	Collider = require 'hardoncollider'
	HC = Collider.new()
	HC:setCallbacks(on_collision)
	
	-- player's blasts
	blastClass = class{ x = 0, y = 0, size = 0 }
	blastClass.__name = 'blast'
	function blastClass:__init ( x, y ,size )
		self.x = x
		self.y = y
		self.size = size
		
		self.collisionmask = HC:addCircle( x, y, size )	
		self.collisionmask.parent = self
	end
	
	function blastClass:setLocation( x, y )
		self.collisionmask:moveTo( x, y )
		self.x = x
		self.y = y		
	end
	
	function blastClass:setSize( size )
		self.collisionmask:scale( size / self.size )
		self.size = size		
	end
	
	function blastClass:clean()
		HC:remove( self.collisionmask )
		self.collisionmask = nil
		
		-- find the blast and remove it
		local i = 1
		for key, value in ipairs( blasts ) do 
			if( value == self ) then
				table.remove( blasts, i )
				return
			end
			i = i + 1
		end		
		
	end

	blasts = {}
	
	-- domes to protect
	domeClass = class{ x = 0, y = 0, image = nil }
	domeClass.__name = 'dome'
	function domeClass:__init( x, y, image )
		self.x = x
		self.y = y
		self.image = image	
		
		self.collisionmask = HC:addRectangle( x, y, image:getWidth(), image:getHeight() )	
		self.collisionmask.parent = self		
	end
	domes = {}
	
	function domeClass:clean()
		HC:remove( self.collisionmask )
		self.collisionmask = nil
		
		-- find the dome and remove it
		local i = 1
		for key, value in ipairs( domes ) do 
			if( value == self ) then
				table.remove( domes, i )
				return
			end
			i = i + 1
		end				
	end
	
	domeImage = love.graphics.newImage( 'images/dome.png' )	
	for i = 1, 5 do
		print( "made dome" )
		table.insert( domes, domeClass:new( (i * 150) - 100, 576, domeImage ) )
	end	
	
	-- rockets
	rocketClass = class{ x = 0, y = 0, rotation = 0, speed = 0, image = nil }
	rocketClass.__name = 'rocket'
	function rocketClass:__init( x, y, rotation, speed, image )
		self.x = x
		self.y = y
		self.rotation = rotation
		self.speed = speed
		self.image = image
				
		self.collisionmask = HC:addRectangle( x, y, image:getWidth(), image:getHeight() )
		self.collisionmask.parent = self
		self.collisionmask:setRotation( rotation )
		
		self.smoketrail = smokeDispenser( x, y, rotation )

	end
	
	function rocketClass:moveTo( x, y )
		self.x = x;
		self.y = y;
				
		-- move collisionmask
		self.collisionmask:moveTo( x , y )
		
		-- move smoketrail
		
		self.smoketrail:setPosition( x - (math.cos( self.rotation )*20), y - (math.sin( self.rotation )*20) )
	end
	
	function rocketClass:clean()
		HC:remove( self.collisionmask )
		self.collisionmask = nil
		
		self.smoketrail = nil
		
		-- find the rocket and remove it
		local i = 1
		for key, value in ipairs( rockets ) do 
			if( value == self ) then
				table.remove( rockets, i )
				return
			end
			i = i + 1
		end		
	end
	
	rocketImage = love.graphics.newImage( 'images/rocket.png' )		
	rockets = {}
	
	-- rocket explosions
	explosionClass = class{ x = 0, y = 0, size = 0 }
	explosionClass.__name = 'explosion'
	function explosionClass:__init( x, y, size )
		self.x = x
		self.y = y
		self.size = size
		self.image = explosionimage
	end
	
	function explosionClass:clean() 
		-- find the explosion and remove it
		local i = 1
		for key, value in ipairs( explosions ) do 
			if( value == self ) then
				table.remove( explosions, i )
				return
			end
			i = i + 1
		end		
	end
	explosions = {}
	
	-- set some globals
	score = 0
	stock = 6
	reload = 0
	spawn = 0
	spawngoal = 1
	goalmin = 1000
	goalmax = 1500
	
	-- draw in white (default)
	love.graphics.setColor( 255, 255, 255, 255 )
	-- background is dark blue
	love.graphics.setBackgroundColor( 7, 35, 99 )
	
	smokeDispenser = require 'smoke'
	
	
	love.filesystem.setIdentity( "highscore" )
	
	-- make sure there's always a highscore file
	if not love.filesystem.exists( "score.data" ) then
		newfile = love.filesystem.newFile( "score.data" )
		newfile:open( "w" )
		newfile:write( "0" )
	end
	
	-- load highscore
	highscorefile = love.filesystem.newFile( "score.data" )	
	highscorefile:open("r")
	highscore = highscorefile:read()
	
end

function love.update( dt )
	-- make blasts larger, remove completed ones
	for key, value in ipairs( blasts ) do
		value:setSize( value.size + (30*dt) )	
		if value.size > 40 then
			value:clean()						
		end
	end

	-- make explosions larger, remove completed ones
	for key, value in ipairs( explosions ) do
		value.size = value.size + (50*dt)
		if value.size > 30 then
			value:clean()						
		end
	end

	-- occasionally spawn new rockets	
	spawn = spawn + dt	
	if( spawn > spawngoal ) then
		table.insert( rockets, rocketClass:new( math.random( 100, 700 ), 0, math.random( 0.25 * math.pi * 100, 0.75 * math.pi * 100 ) / 100, math.random( 50, 100 ), rocketImage ) )
		spawn = spawn - spawngoal
		spawngoal = math.random( goalmin, goalmax ) / 1000
	end
	
	-- spawn rockets faster and faster over time
	if goalmin > 50 then
		goalmin = goalmin - (4*dt)
	end
	if goalmax > 100 then
		goalmax = goalmax - (5*dt)
	end

		
	-- update rocket locations
	for key, value in ipairs( rockets ) do
		value:moveTo( 
			value.x + ( dt * value.speed * math.cos( value.rotation ) ),
			value.y + ( dt * value.speed * math.sin( value.rotation ) )	
		)
		value.smoketrail:update(dt)
		
		-- explode those that hit the ground
		if value.y > 590 then
			explosion = explosionClass( value.x, value.y, math.random(10,20) )
			table.insert( explosions, explosion )
			value:clean()
		end
		-- cleanup outside screen
		if value.x > 825 or value.x < -25 then
			value:clean()
		end
		
		
	end
	
	-- make sure we haven't lost
	if( #domes == 0 ) then
		endgame()		
	end
	
	-- detect collisions
	HC:update(dt)
	
	-- reload ammo
	if stock < 6 then
		reload = reload + dt
		if reload > 1 then
			reload = reload - 1
			stock = stock + 1
		end
	else
		reload = 0
	end
	
	-- time = score
	score = score + 2 * dt
end

function endgame ()
	-- overrule update/draw
	function love.update( dt )	
		-- calculate last few explosions 
		for key, value in ipairs( explosions ) do
			value.size = value.size + (50*dt)
			if value.size > 30 then
				value:clean()						
			end
		end
	end
	
	function love.draw()
		love.graphics.setColor( 255, 255, 255, 255 )
		love.graphics.print( "Game Over", 380, 300 )
		love.graphics.print( "Final score: " .. math.floor( score ), 380, 320 )
		
		-- draw explosions
		love.graphics.setColor( 255, 0, 0, 150 )
		for key, value in ipairs( explosions ) do
			love.graphics.circle( "fill", value.x, value.y, value.size );				
		end
		
		-- draw the ground
		love.graphics.setColor( 8, 196, 11, 255 )
		love.graphics.rectangle( "fill", 0, 595, 800, 5 )

	end
end

-- explode stuff (if there is ammo)
function love.mousepressed( x, y, button )
	if button == 'l' and stock > 0 then
		table.insert( blasts, blastClass:new( x, y, 10 )	 )
		stock = stock - 1
	end
end;

-- quit the game (save highscore if neccesary)
function love.keypressed( button )
	if( button == 'escape' ) then
	
		if math.floor( score ) > tonumber(highscore) then
			scorefile = love.filesystem.newFile( "score.data" )
			scorefile:open( "w" )
			scorefile:write( math.floor( score ) )			
		end
	
		love.event.quit()
	end
end

function love.draw( )
	
	love.graphics.print("Current FPS: "..tostring(love.timer.getFPS( )), 10, 10)
	
	-- draw blasts
	love.graphics.setColor( 255, 255, 255, 150 )
	for key, value in ipairs( blasts ) do
		love.graphics.circle( "fill", value.x, value.y, value.size );		
		--value.collisionmask:draw('fill', 16)
		
	end
	
	-- draw explosions
	love.graphics.setColor( 255, 0, 0, 150 )
	for key, value in ipairs( explosions ) do
		love.graphics.circle( "fill", value.x, value.y, value.size );				
	end
	love.graphics.setColor( 255, 255, 255, 255 )

	-- draw domes
	for key, value in ipairs( domes ) do
		love.graphics.draw( value.image, value.x, value.y )
	end

	-- draw rockets
	for key, value in ipairs( rockets ) do
		love.graphics.draw( value.image, value.x, value.y, value.rotation, 1, 1,  value.image:getWidth()/2, value.image:getHeight()/2 )
		love.graphics.draw( value.smoketrail )
		--value.collisionmask:draw('fill')
	end
	
	-- draw player score
	love.graphics.print( "Score: ", 620, 58 )
	love.graphics.print( math.floor( score ), 699, 58 )	
	love.graphics.print( "Highscore: ", 620, 78 )
	love.graphics.print( highscore, 699, 78 )
	
	-- draw ammo
	love.graphics.print( "Ammo: ", 620, 18 )
	love.graphics.rectangle( "line", 700, 18, 74, 14 )
	for i = 1, stock do
		love.graphics.rectangle( "fill", 702 + 12*(i-1), 20, 10, 10 )
	end
	
	-- draw reloader
	love.graphics.print( "Reload:", 620, 38 )
	love.graphics.rectangle( "line", 700, 40, 54, 9 )
	love.graphics.rectangle( "fill", 702, 42, reload*50, 5 )
	
	-- draw the ground
	love.graphics.setColor( 8, 196, 11, 255 )
	love.graphics.rectangle( "fill", 0, 595, 800, 5 )
	love.graphics.setColor( 255, 255, 255, 255 )
end

function on_collision( dt, shape_a, shape_b, mtv_x, mtv_y )	

	-- collision between blast and rocket	
	if( ( shape_a.parent.__name == 'blast' and shape_b.parent.__name == 'rocket' ) or 
		( shape_b.parent.__name == 'blast' and shape_a.parent.__name == 'rocket' ) ) then
		print( "between rocket and blast" )
		if( shape_a.parent.__name == 'rocket' ) then
			rocket = shape_a.parent
		else
			rocket = shape_b.parent
		end
		
		-- add a point for the player
		score = score + 1
		
		-- draw an explosion
		explosion = explosionClass( rocket.x, rocket.y, math.random(10,20) )
		table.insert( explosions, explosion )
		
		rocket:clean()
		
	end
	
	-- collision between rocket and dome
	if( ( shape_a.parent.__name == 'dome' and shape_b.parent.__name == 'rocket' ) or 
		( shape_b.parent.__name == 'dome' and shape_a.parent.__name == 'rocket' ) ) then
		print( "between rocket and dome" )
		
		if( shape_a.parent.__name == 'dome' ) then
			dome = shape_a.parent
			rocket = shape_b.parent
		else
			rocket = shape_a.parent
			dome = shape_b.parent
		end
		
		-- draw explosions
		for i = 1, 6 do
			explosion = explosionClass( math.random( dome.x, dome.x + 100 ), math.random( dome.y, dome.y + 50 ), math.random( 1, 15 ) )
			table.insert( explosions, explosion )
		end
				
		-- find the dome and rocket and remove them
		dome:clean()
		rocket:clean()
		
	end
end